﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using NationalInstruments.DAQmx;
using System.Diagnostics;

namespace DAQmx_Basics
{
    public partial class DAQmx_Basic : Form
    {
        AnalogSingleChannelReader rschan;
        AnalogSingleChannelWriter wschan;
        AnalogMultiChannelReader reader ;
        AnalogMultiChannelWriter writer ;
        
        Task taskAI ;
        Task taskAO ;

        double[] dAI = new double[2];
        double[] dAO = new double[2];
        bool initOK = false;
        string NoInit = "No Init possible";

        double AD0Display = 0, DA0Display = 0;

        Stopwatch stopwatch1 = new Stopwatch();

        int MinI = -10;
        int MaxI = 10;
        int MinO = -10;
        int MaxO = 10;

 
        public DAQmx_Basic()
        {
            InitializeComponent();
            stopwatch1.Start();
         }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = true;
            this.Hide();
            if (timer1.Enabled)
            {
                //timerAnzeigeZeiten.Stop();
                timer1.Stop();
                dtavr = 0; dtavr2 = 0;
            }
         
        }

        long dtend = 0, dt = 0,  dtavr = 0, cntT0 = 0, dtavr2 = 0;


        private void timer1_Tick(object sender, EventArgs e)
        {
            // This timer writes to DA, so in normal mode this should be disabled
            long dtstart = stopwatch1.ElapsedTicks;

            if (cB2ndChanI.Checked)
            {
                dAI = reader.ReadSingleSample();       //
                dtend = stopwatch1.ElapsedTicks;
                tBAI0.Text = dAI[0].ToString("f3");
                tBAI1.Text = dAI[1].ToString("f3");
                toolStripStatusLabel1.Text = "AD0=  " + dAI[0].ToString("f2") + "  DA0=  " + dAI[1].ToString("F2");
            }
            else
            {
                dAI[0] = rschan.ReadSingleSample();       //
                dtend = stopwatch1.ElapsedTicks;
                tBAI0.Text = dAI[0].ToString("f3");
                toolStripStatusLabel1.Text = "AD0=  " + dAI[0].ToString("f2") ;
            }
            
            dt = (dtend - dtstart);  // Time AD
            dtavr = dtavr + dt;
            dtstart = stopwatch1.ElapsedTicks;
            if (cB2ndChanO.Checked) writer.WriteSingleSample(false, dAO);    //numAO=numericUpDown-Feld
            else wschan.WriteSingleSample(false, dAO[0]);

            dt = (stopwatch1.ElapsedTicks - dtstart);  // Time DA
            dtavr2 = dtavr2 + dt;
            
            cntT0++;
        }
 
        private void timerAnzeigeZeiten_Tick(object sender, EventArgs e)
        {
            if (cntT0 > 0)
            {
                double dtavr1 = dtavr / cntT0 / (double)Stopwatch.Frequency * 1000;
                toolStripStatusLabel2.Text = "DtAD= " + dtavr1.ToString("f3");
                double dtav2 = dtavr2 /cntT0/ (double)Stopwatch.Frequency * 1000;
                toolStripStatusLabel2.Text += " ms  DtDA= " + dtav2.ToString("f3")+ " ms";
                dtavr2 = 0; dtavr = 0; cntT0 = 0;
            }

        }

        private void butInit_Click(object sender, EventArgs e)
        {
            try
            {
                if (!initOK)
                {
                    taskAI = new Task("abci");
                    taskAO = new Task("abco");
                     if (rBRange05V.Checked)
                    { 
                        MinO = 0; MaxO = 5;
                        numAO0.Maximum = MaxO;
                        numAO1.Maximum = MaxO;
                        numAO0.Minimum = MinO;
                        numAO1.Minimum = MinO;
                    }
                    string actdev = comboBox1.Text;
                    taskAI.AIChannels.CreateVoltageChannel(actdev + "/ai0", "aiChannel0",
                         AITerminalConfiguration.Rse, MinI, MaxI, AIVoltageUnits.Volts);
                    if (cB2ndChanI.Checked) taskAI.AIChannels.CreateVoltageChannel(actdev + "/ai1", "aiChannel1",
                        AITerminalConfiguration.Rse, MinI, MaxI, AIVoltageUnits.Volts);
                    taskAO.AOChannels.CreateVoltageChannel(actdev+"/ao0", "aoChannel0",
                        MinO, MaxO, AOVoltageUnits.Volts);
                    if (cB2ndChanO.Checked) taskAO.AOChannels.CreateVoltageChannel(actdev + "/ao1", "aoChannel1",
                        MinO, MaxO, AOVoltageUnits.Volts);

                    if (cB2ndChanI.Checked) reader = new AnalogMultiChannelReader(taskAI.Stream);
                    else rschan = new AnalogSingleChannelReader(taskAI.Stream);

                    if (cB2ndChanO.Checked) writer = new AnalogMultiChannelWriter(taskAO.Stream);
                    else wschan = new AnalogSingleChannelWriter(taskAO.Stream);
                    taskAI.Start();
                    taskAO.Start();


                    timer1.Start(); 
                    timerAnzeigeZeiten.Enabled = true;
                    initOK = true;
                    label5NOK.Text = "OK";
                    dAO[1] = 0;
                }
                else timer1.Start();
            }
            catch
            {
                MessageBox.Show(NoInit);
            }
        }
 
        private void DAQmx_Basic_FormClosed(object sender, FormClosedEventArgs e)
        {
                taskAI.Dispose();
                taskAO.Dispose(); 
        }
 
        public void InputNImx(short chan, ref double val)
        {
            if (initOK)
                if (cB2ndChanI.Checked) dAI = reader.ReadSingleSample();
                else dAI[0] = rschan.ReadSingleSample();
            if (chan == 0) val = dAI[0];
            if (chan == 1) if (cB2ndChanI.Checked) val = dAI[1];
            AD0Display = val;
        }

        private void numAO0_ValueChanged(object sender, EventArgs e)
        {
            dAO[0] = (double)numAO0.Value;
            if (initOK)
                if (cB2ndChanO.Checked) writer.WriteSingleSample(false, dAO);    //numAO=numericUpDown-Feld
                else wschan.WriteSingleSample(true, dAO[0]);

        }

        private void numAO1_ValueChanged(object sender, EventArgs e)
        {
            dAO[1] = (double)numAO1.Value;
            if (initOK) writer.WriteSingleSample(false, dAO);    //numAO=numericUpDown-Feld

        }

        public void OutputNImx(short chan, double val)
        {
            if (val > MaxO) val = MaxO;
            if (val < MinO) val = MinO;
            if (chan == 0) dAO[0] = val;
            if (chan == 1) if (cB2ndChanO.Checked) dAO[1] = val;
            if (initOK)
                if (cB2ndChanO.Checked) writer.WriteSingleSample(false, dAO);    //numAO=numericUpDown-Feld
                else wschan.WriteSingleSample(false, dAO[0]);
            DA0Display = val;

        }

        private void cB2ndChanI_CheckedChanged(object sender, EventArgs e)
        {
            if (cB2ndChanI.Checked) { label1.Visible = true; tBAI1.Visible = true; }
            else { label1.Visible = false; tBAI1.Visible = false; }
        }

        private void cB2ndChanO_CheckedChanged(object sender, EventArgs e)
        {
            if (cB2ndChanO.Checked) { label2.Visible = true; numAO1.Visible = true; }
            else { label2.Visible = false; numAO1.Visible = false; }
        }

 
    }
}
